/* Commands:
 
 ***vent1 0x1n***
 * 0x10 Vent1 off 
 * 0x11 Vent1 100% 
 * 0x12 Vent1 20%  
 * ...
 * 0x19 Vent1 90%
 ***vent2 0x2n***
 * 0x20 Vent1 off
 * 0x21 Vent1 100%
 * 0x22 Vent1 20%
 * ...
 * 0x29 Vent1 90%
 ***temp sensors 0x3n***  
 * 0x31 get Temp1 (4 values)
 * 0x32 get Temp2 (4 values)
 * 0x33 get Temp3 (4 values)
 * 0x34 get Temp4 (4 values) 
 
 ***tacho 0x4n***  
 * 0x41 get Tacho1
 * 0x42 get Tacho2
 
*/

//https://github.com/MikeOchtman/Pi_Arduino_I2C/blob/master/Arduino_I2C.ino#L89

#include <Wire.h>
#include "Sensirion.h"

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif

// Sensirion(dataPin, sclkPin, address, noholdmaster);
Sensirion sht1 = Sensirion(2, 1);   // clk1  PB1, data1 PB2 
Sensirion sht2 = Sensirion(3, 4);   // clk2  PB7, data2 PB3 
Sensirion sht3 = Sensirion(9, 12);   // clk3  PD7, data3 PC6 
Sensirion sht4 = Sensirion(16, 13); // clk4  PB4, data4 PF7 

// Fortluft
const byte fan1Pin = 10;    // PC7 on Teensy (OC4A of Timer 4)
const byte Tacho1Pin = 14;  // PB5 on Teensy (PCINT5)
const byte PCINTPinT1 = 5;  // PCINT5
// Aussenluft (Frischluft)
const byte fan2Pin = 15;    // PB6 on Teensy (OC4B of Timer 4)
const byte Tacho2Pin = 0;   // PB0 on Teensy (PCINT0)
const byte PCINTPinT2 = 0;  // PCINT0

const byte teensyLED = 11; // PD6 Teensy LED
const byte teensyI2CAddr = 0x20;

volatile byte oldPort = 0x00;
volatile byte T1Cnt = 0, T2Cnt = 0;
volatile byte T1CntVal = 0, T2CntVal = 0;
byte volatile Command, Control, Sensor, Tacho;
byte volatile txTable[10], rxTable[10]; 

struct {
  float t1, t2, t3, t4;
  float h1,h2,h3,h4;
  float dp1,dp2,dp3,dp4;
  float h371,h372,h373,h374;  
  int ret1,ret2,ret3,ret4;
} t;

struct {
  float volatile t1, t2, t3, t4;
  float volatile h1,h2,h3,h4;
  float volatile dp1,dp2,dp3,dp4;
  float volatile h371,h372,h373,h374;  
} data;

void setup() {
  Command = 0;
  Control = 0;
  Sensor = 0;
  Tacho = 0;
  OCR4A = 127; // both vents on 50%
  OCR4B = 127; // inverted because of Transistor!
  pinMode(teensyLED, OUTPUT);
  pinMode(fan1Pin, OUTPUT);
  pinMode(fan2Pin, OUTPUT);
  // Fast PWM for Teensy Timer 4
  TCCR4A = _BV(COM4A1) | _BV(COM4B1) | _BV(PWM4A) | _BV(PWM4B);
  TCCR4B = _BV(CS42); //3,9kHz
  Wire.begin(teensyI2CAddr);    // join i2c bus
  cbi(PORTD, 0); // no internal PU
  cbi(PORTD, 1); // Raspi has 2x1,8k
  Wire.onReceive(i2cReceive); // i2c interrupt receive
  Wire.onRequest(i2cTransmit); // i2c interrupt send  
  // set pins to input with a pullup
  pinMode(Tacho1Pin, INPUT_PULLUP);
  pinMode(Tacho2Pin, INPUT_PULLUP);
   // attach the PinChangeInterrupts
  attachPinChangeInterrupt();
  //Serial.begin(9600);  // for debugging
}

void loop() {
  while (!(t.ret1 == S_Meas_Rdy)) {// A new measurement is available
    t.ret1 = sht1.measure(&t.t1, &t.h1, &t.dp1, 37, &t.h371);
    //Serial.print(t.ret1);
    }
  data.t1 = t.t1;  
  //Serial.print(data.t1);
  data.h1 = t.h1;
  data.dp1 = t.dp1;
  data.h371 = t.h371; 
  t.ret1 = 0; 
  while (!(t.ret2 == S_Meas_Rdy)) {
    t.ret2 = sht2.measure(&t.t2, &t.h2, &t.dp2, 37, &t.h372);} // Measurement
  data.t2 = t.t2;
  data.h2 = t.h2;
  data.dp2 = t.dp2;
  data.h372 = t.h372;
  t.ret2 = 0; 
  while (!(t.ret3 == S_Meas_Rdy)) {
    t.ret3 = sht3.measure(&t.t3, &t.h3, &t.dp3, 37, &t.h373);} // Measurement
  data.t3 = t.t3;
  data.h3 = t.h3;
  data.dp3 = t.dp3;
  data.h373 = t.h373;          
  t.ret3 = 0; 
  while (!(t.ret4 == S_Meas_Rdy)) {
    t.ret4 = sht4.measure(&t.t4, &t.h4, &t.dp4, 37, &t.h374);} // Measurement
  data.t4 = t.t4;
  data.h4 = t.h4;
  data.dp4 = t.dp4;
  data.h374 = t.h374;          
  t.ret4 = 0; 
  //int iaq1 = analogRead(0);
  //int iaq2 = analogRead(1);
  //Serial.print("H1:"); 
  //Serial.print(iaq1); 
  //Serial.print(" H2:"); 
  //Serial.println(iaq2); 
  
  digitalWrite(teensyLED,LOW);  
  T1Cnt = 0;
  T2Cnt = 0;
  for (unsigned int i=0; i <= 100; i++) delayMicroseconds(10000); // measure during 1s
  T1CntVal = T1Cnt;
  T2CntVal = T2Cnt;
  digitalWrite(teensyLED,HIGH);
  for (unsigned int i=0; i <= 10; i++) delayMicroseconds(10000); // 100ms*/
}


void i2cReceive(int byteCount) {
  while(1 < Wire.available()) {
    char c = Wire.read(); 
  }
  Command = Wire.read();  
  // parse vent commands (0x1n, 0x2n)
  switch (Command) {        
    case 0x10:
      OCR4A = 255;  // off
      break;
    case 0x11:
      OCR4A = 1;    // 100%
      break;
    case 0x12:
      OCR4A = 225;  // 20%
      break;
    case 0x13:
      OCR4A = 215;  // 30%
      break;
    case 0x14:
      OCR4A = 200;  // 40%
      break;
    case 0x15:
      OCR4A = 178;  // 50%
      break;
    case 0x16:
      OCR4A = 153;  // 60%
      break;
    case 0x17:
      OCR4A = 130;  // 70%
      break;
    case 0x18:
      OCR4A = 90;   // 80%
      break;
    case 0x19:
      OCR4A = 60;   // 90%
      break;
    case 0x20:
      OCR4B = 255;  // off
      break;
    case 0x21:
      OCR4B = 1;    // 100%
      break;
    case 0x22:
      OCR4B = 225;  // 20%
      break;
    case 0x23:
      OCR4B = 215;  // 30%
      break;
    case 0x24:
      OCR4B = 200;  // 40%
      break;
    case 0x25:
      OCR4B = 178;  // 50%
      break;
    case 0x26:
      OCR4B = 153;  // 60%
      break;
    case 0x27:
      OCR4B = 130;  // 70%
      break;
    case 0x28:
      OCR4B = 90;   // 80%
      break;
    case 0x29:
      OCR4B = 60;   // 90%
      break;
    case 0x31:
      Sensor = 1;
      Control = 8;
      break;
    case 0x32:
      Sensor = 2;
      Control = 8;
      break;
    case 0x33:
      Sensor = 3;
      Control = 8;
      break;
    case 0x34:
      Sensor = 4;
      Control = 8;
      break;
    case 0x41:
      Tacho = 1;
      Control = 1;
      break;
    case 0x42:
      Tacho = 2;
      Control = 1;
      break;
    default:  
      break;
  }
  Command = 0;
}  

void i2cTransmit() {      
  int tmp1 = 0; // temporary variable used in switch occasionally below
  if (Control == 0) return;
  Control -= 1;
  switch (Sensor) {
    case 1: // 
      tmp1 = int(round(data.t1 * 100));            
      txTable[6] = (byte)(tmp1 & 0xFF);      
      txTable[7] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h1 * 100));
      txTable[4] = (byte)(tmp1 & 0xFF);
      txTable[5] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.dp1 * 100));
      txTable[2] = (byte)(tmp1 & 0xFF);
      txTable[3] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h371 * 100));
      txTable[0] = (byte)(tmp1 & 0xFF);
      txTable[1] = (byte)(tmp1 >> 8);
      break;
    case 2: // 
      tmp1 = int(round(data.t2 * 100));
      txTable[6] = (byte)(tmp1 & 0xFF);
      txTable[7] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h2 * 100));
      txTable[4] = (byte)(tmp1 & 0xFF);
      txTable[5] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.dp2 * 100));
      txTable[2] = (byte)(tmp1 & 0xFF);
      txTable[3] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h372 * 100));
      txTable[0] = (byte)(tmp1 & 0xFF);
      txTable[1] = (byte)(tmp1 >> 8);
      break;
    case 3: // 
      tmp1 = int(round(data.t3 * 100));
      txTable[6] = (byte)(tmp1 & 0xFF);
      txTable[7] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h3 * 100));
      txTable[4] = (byte)(tmp1 & 0xFF);
      txTable[5] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.dp3 * 100));
      txTable[2] = (byte)(tmp1 & 0xFF);
      txTable[3] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h373 * 100));
      txTable[0] = (byte)(tmp1 & 0xFF);
      txTable[1] = (byte)(tmp1 >> 8);
      break;
    case 4: // 
      tmp1 = int(round(data.t4 * 100));
      txTable[6] = (byte)(tmp1 & 0xFF);
      txTable[7] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h4 * 100));
      txTable[4] = (byte)(tmp1 & 0xFF);
      txTable[5] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.dp4 * 100));
      txTable[2] = (byte)(tmp1 & 0xFF);
      txTable[3] = (byte)(tmp1 >> 8);
      tmp1 = int(round(data.h374 * 100));
      txTable[0] = (byte)(tmp1 & 0xFF);
      txTable[1] = (byte)(tmp1 >> 8);
      break;
    default:
      break; }
  if (Tacho == 1) { txTable[0] = T1CntVal; }   
  else {
    if (Tacho == 2) { txTable[0] = T2CntVal;}}   
  Wire.write(txTable[Control]); }

void attachPinChangeInterrupt(void) {  
  oldPort = PINB;
  // pin change mask registers decide which pins are enabled as triggers
  PCMSK0 |= (1 << PCINTPinT1); //PCINT0
  PCMSK0 |= (1 << PCINTPinT2); //PCINT5
  // PCICR: Pin Change Interrupt Control Register - enables interrupt vectors
  PCICR |= (1 << PCIE0); }

ISR(PCINT0_vect) {
  byte newPort = PINB;             // get the new pin states
  byte change = newPort ^ oldPort; // xor to detect a rising or falling
  // check which pins are triggered, compared with the settings
  byte rising = change & newPort;
  byte mask1 = (1 << PCINTPinT1);
  if (rising & mask1) T1Cnt++;
  byte mask2 = (1 << PCINTPinT2);
  if (rising & mask2) T2Cnt++;
  oldPort = newPort; }  // save the new state for next comparison 




